@@ -2,7 +2,7 @@
 
 import re
 import django
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import ugettext as _
 
 
 def unescape(text):
@@ -37,6 +37,7 @@ def get_user_model():
         return get_user_model()
     else:
         from django.contrib.auth.models import User
+        User.get_username = lambda u: u.username  # emulate new 1.5 method
         return User
 
 
